/*
Copyright © 2021 NAME HERE <EMAIL ADDRESS>

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/
package cmd

import (
	"log"

	"github.com/spf13/cobra"
	"libre.sh/cli/pkg/cluster"
)

var provisioner string
var configFile string
var dryRUn bool
var stage string

var clusterCmd = &cobra.Command{
	Use:   "cluster",
	Short: "cluster mangement",
	Long:  ``,
}

var clusterApplyCmd = &cobra.Command{
	Use:   "apply",
	Short: "apply cluster config",
	Long:  ``,
	Run: func(cmd *cobra.Command, args []string) {
		c := cluster.New(provisioner, configFile, dryRUn)
		if err := c.ApplyStage(stage); err != nil {
			log.Fatal(err)
		}
	},
}

var clusterSshConfigCmd = &cobra.Command{
	Use:   "ssh-config",
	Short: "print cluster ssh config",
	Long:  ``,
	Run: func(cmd *cobra.Command, args []string) {
		c := cluster.New(provisioner, configFile, dryRUn)
		if err := c.PrintSSHConfig(); err != nil {
			log.Fatal(err)
		}
	},
}

func init() {
	rootCmd.AddCommand(clusterCmd)
	clusterCmd.AddCommand(clusterApplyCmd)
	clusterCmd.AddCommand(clusterSshConfigCmd)
	clusterCmd.PersistentFlags().StringVar(&provisioner, "provisioner", "ada-lovelace", "Cluster provisioner")
	clusterCmd.PersistentFlags().StringVar(&configFile, "config", "config.yaml", "Config file path")
	clusterCmd.PersistentFlags().BoolVar(&dryRUn, "dry-run", false, "Don't take actions on the infra")
	clusterCmd.PersistentFlags().StringVar(&stage, "stage", "all", "Specify a stage to run")
}
