package adalovelace

import (
	"os"

	"github.com/thanhpk/randstr"
)

type Config struct {
	Name     string          `json:"name"`
	SSHKeys  []string        `json:"sshKeys"`
	Location string          `json:"location"`
	Network  Network         `json:"network"`
	API      API             `json:"api"`
	Ingress  Ingress         `json:"ingress"`
	Firewall Firewall        `json:"firewall"`
	Nodes    map[string]Node `json:"nodes"`
	Services Services        `json:"services"`
}

type Network struct {
	ID        string `json:"id"`
	Zone      string `json:"zone"`
	CIDR      string `json:"cidr"`
	CloudCIDR string `json:"cloudCidr"`
	PhysCIDR  string `json:"physCidr"`
	VSwitchID int    `json:"vSwitchId"`
}

type API struct {
	LBType    string `json:"lbType"`
	PrivateIP string `json:"privateIp"`
	PublicIP  string `json:"publicIp"`
}

type Ingress struct {
	LBType    string         `json:"lbType"`
	PrivateIP string         `json:"privateIp"`
	PublicIP  string         `json:"publicIp"`
	Ports     map[int]string `json:"ports"`
}

type Firewall struct {
	Allow map[string]FirewallRule `json:"allow"`
}

type FirewallRule struct {
	Port  int    `json:"port"`
	Proto string `json:"proto"`
}

type Node struct {
	Type        string            `json:"type"`
	Role        string            `json:"role"`
	ServerType  string            `json:"serverType"`
	PrivateIP   string            `json:"privateIp"`
	PublicIP    string            `json:"publicIp"`
	Ingress     bool              `json:"ingress"`
	IngressOnly bool              `json:"ingressOnly"`
	Backups     bool              `json:"backups"`
	Volumes     map[string]Volume `json:"volumes"`
}

type Volume struct {
	Size   int    `json:"size"`
	VG     bool   `json:"vg"`
	Device string `json:"device"`
}

type Services struct {
	CertManager     ServiceBase     `json:"certManager"`
	NginxIngress    ServiceBase     `json:"nginxIngress"`
	PrometheusStack PrometheusStack `json:"prometheusStack"`
	LVMLocalPV      ServiceBase     `json:"lvmLocalPv"`
}

type ServiceBase struct {
	Skip    bool   `json:"skip"`
	Version string `json:"version"`
}

type PrometheusStack struct {
	ServiceBase
	GrafanaHost string `json:"grafanaHost"`
}

func (c *Config) SetDefaults() {
	if c.Services.CertManager.Version == "" {
		c.Services.CertManager.Version = "1.4.2"
	}

	if c.Services.NginxIngress.Version == "" {
		c.Services.NginxIngress.Version = "3.35.0"
	}

	if c.Services.PrometheusStack.Version == "" {
		c.Services.PrometheusStack.Version = "17.1.0"
	}

	if c.Services.LVMLocalPV.Version == "" {
		c.Services.LVMLocalPV.Version = "0.7.2"
	}
}

func (c *Config) GrafanaPassword() string {
	pass := os.Getenv("GRAFANA_PASSWORD")
	if pass == "" {
		pass = randstr.String(16)
	}
	return pass
}
