package adalovelace

import (
	"bytes"
	"io/ioutil"
	"os"
	"os/exec"
	"strings"
	"text/template"

	"github.com/Masterminds/sprig"
)

func (p *Provisioner) BootstrapStage() error {
	if err := refreshK0sFolder(); err != nil {
		return err
	}

	if err := generatingK0sConfig(&p.Config); err != nil {
		return err
	}

	if !p.DryRun {
		if err := k0sctlApply(); err != nil {
			return err
		}
	}

	if err := k0sctlKubeconfig(&p.Config); err != nil {
		if !p.DryRun {
			return err
		}
	}

	return nil
}

func refreshK0sFolder() error {
	if err := os.RemoveAll("k0s"); err != nil && !os.IsNotExist(err) {
		return err
	}

	err := os.Mkdir("k0s", 0775)
	if err != nil && !os.IsExist(err) {
		return err
	}
	return nil
}

func generatingK0sConfig(c *Config) error {
	tpl, err := template.New("k0sctl.yaml").Funcs(sprig.TxtFuncMap()).ParseFS(tplFS, "templates/k0s/k0sctl.yaml")
	if err != nil {
		return err
	}

	f, err := os.OpenFile("k0s/"+tpl.Name(), os.O_CREATE|os.O_WRONLY, 0664)
	if err != nil {
		return err
	}
	defer f.Close()

	return tpl.Execute(f, c)
}

func k0sctlApply() error {
	cmd := exec.Command("k0sctl", "apply", "--disable-telemetry", "--no-drain")
	cmd.Stdout = os.Stdout
	cmd.Stderr = os.Stderr
	cmd.Dir = "k0s"
	return cmd.Run()
}

func k0sctlKubeconfig(c *Config) error {
	cmd := exec.Command("k0sctl", "kubeconfig", "--disable-telemetry")
	var out bytes.Buffer
	cmd.Stdout = &out
	cmd.Stderr = os.Stderr
	cmd.Dir = "k0s"

	if err := cmd.Run(); err != nil {
		return err
	}

	kubeconfig := strings.ReplaceAll(out.String(), c.API.PrivateIP, c.API.PublicIP)

	return ioutil.WriteFile("k0s/kubeconfig.yaml", []byte(kubeconfig), os.FileMode(0600))
}
