package adalovelace

import (
	"bytes"
	"fmt"
	"os"
	"os/exec"
	"regexp"

	"github.com/Masterminds/semver"
)

type Version struct {
	Command    string
	Args       []string
	Regex      string
	Constraint string
}

func getVersion(v Version) (*semver.Version, error) {
	cmd := exec.Command(v.Command, v.Args...)
	var out bytes.Buffer
	cmd.Stdout = &out
	cmd.Stderr = os.Stderr

	if err := cmd.Run(); err != nil {
		return nil, err
	}

	re := regexp.MustCompile(v.Regex)
	matches := re.FindStringSubmatch(out.String())
	if len(matches) < 2 {
		return nil, fmt.Errorf("%s version not found", v.Command)
	}

	return semver.NewVersion(matches[1])
}

func checkVersion(v Version) error {
	version, err := getVersion(v)
	if err != nil {
		return err
	}
	c, _ := semver.NewConstraint(v.Constraint)
	if check, err := c.Validate(version); !check {
		return fmt.Errorf("%s version %s", v.Command, err[0])
	}
	return nil
}

func CheckVersions(versions []Version) error {
	for _, v := range versions {
		if err := checkVersion(v); err != nil {
			return err
		}
	}
	return nil
}
