package cluster

import (
	"log"

	"libre.sh/cli/pkg/adalovelace"
)

type Provisioner interface {
	ApplyStage(string) error
	// Destroy() error
	ListStages() []string

	CheckRequirements() error
	PrintSSHConfig() error
}

type ClusterProvisioner struct {
	Provisioner
}

func New(provisioner string, configPath string, dryRun bool) Provisioner {
	var cluster ClusterProvisioner
	switch provisioner {
	case "ada-lovelace":
		cluster.Provisioner = adalovelace.New(configPath, dryRun)
	default:
		log.Fatal("unkown cluster provisioner")
	}

	return cluster
}

func (c ClusterProvisioner) ApplyStage(name string) error {
	if err := c.Provisioner.CheckRequirements(); err != nil {
		return err
	}
	if name == "all" {
		for _, stage := range c.Provisioner.ListStages() {
			if err := c.Provisioner.ApplyStage(stage); err != nil {
				return err
			}
		}
		return nil
	}

	return c.Provisioner.ApplyStage(name)
}
